using System;
using System.IO;
using System.Security;
using System.Security.Cryptography;
using System.Text;

namespace Tanuki {
	class MainClass {
			static void Main(string[] args) {
			if (args.Length != 3) {
				Console.WriteLine("You must specify either + or - as the first parameter, then input and output files as the second and third parameters.");
				return;
			}

			Console.Write("Please enter your key: ");
			string key = Console.ReadLine();
			if (string.IsNullOrEmpty(key)) {
				Console.WriteLine("No key provided - assuming 'frosties'");
				key = "frosties";
			}

			key = Sha1(key);

			if (!File.Exists(args[1])) {
				Console.WriteLine("Input file doesn't exist!");
				return;
			}

			if (args[0] == "+") {
				EncryptFile(args[1], args[2], key);
			} else if (args[0] == "-") {
				DecryptFile(args[1], args[2], key);
			} else {
				Console.WriteLine("Please specify either + or - as the first parameter.");
			}
		}
	
		static void EncryptFile(string file_in, string file_out, string key) {
			try {
				FileStream output = new FileStream(file_out, FileMode.Create, FileAccess.Write);

				RijndaelManaged aes = new RijndaelManaged();
				aes.Key = Encoding.UTF8.GetBytes(key.Substring(0, 32));
				aes.IV = Encoding.UTF8.GetBytes(key.Substring(5, 16));

				ICryptoTransform transform = aes.CreateEncryptor();
				CryptoStream cryptostream = new CryptoStream(output, transform, CryptoStreamMode.Write);

				byte[] inputbytes = File.ReadAllBytes(file_in);
				cryptostream.Write(inputbytes, 0, inputbytes.Length);
				cryptostream.Close();
				output.Close();
			} catch (Exception e) {
				Console.WriteLine(e.Message);
			}
		}

		static void DecryptFile(string file_in, string file_out, string key) {
			try {
				FileStream input = new FileStream(file_in, FileMode.Open, FileAccess.Read);
				StreamWriter output = new StreamWriter(file_out);

				RijndaelManaged aes = new RijndaelManaged();
				aes.Key = Encoding.UTF8.GetBytes(key.Substring(0, 32));
				aes.IV = Encoding.UTF8.GetBytes(key.Substring(5, 16));

				ICryptoTransform transform = aes.CreateDecryptor();
				CryptoStream cryptostream = new CryptoStream(input, transform, CryptoStreamMode.Read);

				output.Write(new StreamReader(cryptostream).ReadToEnd());
				output.Flush();
				output.Close();
			} catch (Exception e) {
				Console.WriteLine(e.Message);
			}
		}

		static string Sha1(string input) {
			SHA1Managed hashgen = new SHA1Managed();
			byte[] hash = hashgen.ComputeHash(Encoding.UTF8.GetBytes(input));
			string result = "";
			foreach (byte b in hash) result += b.ToString("x2");
			return result;
		}
	}
}